<?php

require_once "phpfspot_cfg.php";
require_once "phpfspot_db.php";
require_once "phpfspot_tmpl.php";

class PHPFSPOT {

   var $cfg;
   var $db;
   var $cfg_db;
   var $tmpl;
   var $tags;
   var $avail_tags;
   var $current_tags;

   public function __construct()
   {
      $this->cfg = new PHPFSPOT_CFG;

      $this->db = new PHPFSPOT_DB(&$this, $this->cfg->fspot_db);

      $this->cfg_db = new PHPFSPOT_DB(&$this, $this->cfg->phpfspot_db);
      $this->check_config_table();

      $this->tmpl = new PHPFSPOT_TMPL($this);

      $this->get_tags();

   } // __construct()

   public function __destruct()
   {

   } // __destruct()

   public function show()
   {
      $this->tmpl->assign('page_title', $this->cfg->page_title);
      $this->tmpl->show("index.tpl");

   } // show()

   private function get_tags()
   {
      $this->avail_tags = Array();
      $count = 0;
   
      $result = $this->db->db_query("
         SELECT id,name
         FROM tags
         ORDER BY sort_priority ASC
      ");
      
      while($row = $this->db->db_fetch_object($result)) {

         $tag_id = $row['id'];
         $tag_name = $row['name'];

         $this->tags[$tag_id] = $tag_name; 
         $this->avail_tags[$count] = $tag_id;

         $count++;

      }

   } // get_tags()

   public function get_photo_details($idx)
   {
      $result = $this->db->db_query("
         SELECT *
         FROM photos
         WHERE id='". $idx ."'
      ");
      
      return $this->db->db_fetch_object($result);

   } // get_photo_details

   public function translate_path($path, $width = 0)
   {  
      return str_replace($this->cfg->path_replace_from, $this->cfg->path_replace_to, $path);

   } // translate_path

   public function showPhoto($photo)
   {
      $all_photos = $this->getAllTagPhotos();

      foreach($all_photos as $all_photo) {
         
         if($get_next) {
            $next_img = $all_photo;
            break;
         }

         if($all_photo == $photo) {
            $get_next = 1;
         }
         else {
            $previous_img = $all_photo;
         }
      }


      $details = $this->get_photo_details($photo);
      $meta = $this->get_meta_informations($this->translate_path($details['directory_path']) ."/". $details['name']);
      $info = getimagesize($this->translate_path($details['directory_path']) ."/thumbs/". $this->cfg->photo_width ."_". $details['name']);

      $this->tmpl->assign('width', $info[0]);
      $this->tmpl->assign('height', $info[1]);
      $this->tmpl->assign('c_date', $meta['DateTime']);
      $this->tmpl->assign('madewith', $meta['Make'] ." ". $meta['Model']);
      $this->tmpl->assign('image_name', $details['name']);
      $this->tmpl->assign('image_url', 'phpfspot_img.php?idx='. $photo ."&amp;width=". $this->cfg->photo_width);
      $this->tmpl->assign('image_url_full', 'phpfspot_img.php?idx='. $photo);

      if($previous_img) {
         $this->tmpl->assign('previous_url', "javascript:showImage(". $previous_img .");");
      }

      if($next_img) {
         $this->tmpl->assign('next_url', "javascript:showImage(". $next_img .");");
      }

      $this->tmpl->show("single_photo.tpl");

   } // showPhoto()

   public function getAvailableTags()
   {
      foreach($this->avail_tags as $tag)
      {
         if(isset($_SESSION['selected_tags']) && in_array($tag, $_SESSION['selected_tags']))
            continue;

         // return all available (= not selected) tags
         print "<a href=\"javascript:Tags('add', ". $tag .");\">". $this->tags[$tag] ."</a>&nbsp;";

      }

   } // getAvailableTags()

   public function getSelectedTags()
   {
      foreach($this->avail_tags as $tag)
      {
         // return all selected tags
         if(isset($_SESSION['selected_tags']) && in_array($tag, $_SESSION['selected_tags'])) {
            print "<a href=\"javascript:Tags('del', ". $tag .");\">". $this->tags[$tag] ."</a>&nbsp;";
         }

      }

   } // getSelectedTags()

   public function addTag($tag)
   {
      if(!isset($_SESSION['selected_tags']))
         $_SESSION['selected_tags'] = Array();

      array_push($_SESSION['selected_tags'], $tag);
   
   } // addTag()

   public function delTag($tag)
   {
      if(isset($_SESSION['selected_tags'])) {
         $key = array_search($tag, $_SESSION['selected_tags']);
         unset($_SESSION['selected_tags'][$key]);
      }

   } // delTag()

   public function resetTags()
   {
      unset($_SESSION['selected_tags']);

   } // resetTags()

   public function getAllTagPhotos()
   {  
      $tagged_photos = Array();

      if(isset($_SESSION['selected_tags'])) {
         $selected = "";
         foreach($_SESSION['selected_tags'] as $tag)
            $selected.= $tag .",";
         $selected = substr($selected, 0, strlen($selected)-1);
         $result = $this->db->db_query("
            SELECT DISTINCT photo_id
               FROM photo_tags pt
            INNER JOIN photos p
               ON p.id=pt.photo_id
            WHERE pt.tag_id IN (". $selected .")
            ORDER BY p.time ASC
         ");
      }
      else {
         $result = $this->db->db_query("
            SELECT DISTINCT photo_id
               FROM photo_tags pt
            INNER JOIN photos p
               ON p.id=pt.photo_id
            ORDER BY p.time ASC
         ");
      }

      while($row = $this->db->db_fetch_object($result)) {
         array_push($tagged_photos, $row['photo_id']);
      }

      return $tagged_photos;

   } // getAllTagPhotos()

   public function showPhotoIndex()
   {
      $photos = $this->getAllTagPhotos();
      $count = count($photos);

      $rows = 0;
      $cols = 0;
      $images[$rows] = Array();

      for($i = 0; $i < $count; $i++) {

         $images[$rows][$cols] = $photos[$i];

         if($cols == $this->cfg->thumbs_per_row-1) {
            $cols = 0;
            $rows++;
            $images[$rows] = Array();
         }
         else {
            $cols++;
         }
      } 

      // +1 for for smarty's selection iteration
      $rows++;

         //$images.= "<img src=\"phpfspot_img.php?idx=". $photo ."&amp;width=". $this->cfg->thumb_width ."\" /><br />\n";

      $this->tmpl->assign('count', $count);
      $this->tmpl->assign('width', $this->cfg->thumb_width);
      $this->tmpl->assign('images', $images);
      $this->tmpl->assign('rows', $rows);
      $this->tmpl->assign('columns', $this->cfg->thumbs_per_row);
      $this->tmpl->show("photo_index.tpl");


   } // showPhotoIndex()

   public function showBubbleDetails($photo, $direction)
   {
      if($direction == "up")
         $direction = "bubbleimg_up";
      else
         $direction = "bubbleimg_down";

      $details = $this->get_photo_details($photo);

      $image_url = "phpfspot_img.php?idx=". $photo ."&amp;width=200";

      $filesize = filesize($this->translate_path($details['directory_path'])  ."/". $details['name']);
      $filesize = rand($filesize/1024, 2);

      $img = getimagesize($this->translate_path($details['directory_path'])  ."/". $details['name']);

      $this->tmpl->assign('file_size', $filesize);
      $this->tmpl->assign('width', $img[0]);
      $this->tmpl->assign('height', $img[1]);
      $this->tmpl->assign('file_name', $details['name']);
      $this->tmpl->assign('image_id', $direction);
      $this->tmpl->assign('image_url', $image_url);
      $this->tmpl->show("bubble_details.tpl");

   } // showBubbleDetails()

   public function showCredits()
   {
      $this->tmpl->assign('version', $this->cfg->version);
      $this->tmpl->assign('product', $this->cfg->product);
      $this->tmpl->show("credits.tpl");

   } // showCredits()

   public function create_thumbnail($image, $width)
   {  
      // if thumbnail already exists, don't recreate it
      if(file_exists(dirname($image) ."/thumbs/". $width ."_". basename($image)))
         return;

      $src_img = @imagecreatefromjpeg($image);

      if($src_img)
      {  
         /* grabs the height and width */
         $new_w = imagesx($src_img);
         $new_h = imagesy($src_img);

         // If requested width is more then the actual image width,
         // do not generate a thumbnail

         if($width >= $new_w) {
            imagedestroy($src_img);
            return;
         }

         /* calculates aspect ratio */
         $aspect_ratio = $new_h / $new_w;

         /* sets new size */
         $new_w = $width;
         $new_h = abs($new_w * $aspect_ratio);

         /* creates new image of that size */
         $dst_img = imagecreatetruecolor($new_w,$new_h);

         imagefill($dst_img, 0, 0, ImageColorAllocate($dst_img, 255, 255, 255));

         /* copies resized portion of original image into new image */
         imagecopyresampled($dst_img,$src_img,0,0,0,0,$new_w,$new_h,imagesx($src_img),imagesy($src_img));

         /* write down new generated file */

         if(!file_exists(dirname($image) ."/thumbs"))
            mkdir(dirname($image) ."/thumbs");

         $newfile = dirname($image) ."/thumbs/". $width ."_". basename($image);
         imagejpeg($dst_img, $newfile, 75);

         /* free your mind */
         imagedestroy($dst_img);
         imagedestroy($src_img);
      }

   } // create_thumbnail()

   public function get_meta_informations($file)
   {

      return exif_read_data($file);

   } // get_meta_informations()

   public function check_config_table()
   {
      // if the config table doesn't exist yet, create it
      if(!$this->cfg_db->db_check_table_exists("images")) {
         $this->cfg_db->db_exec("
            CREATE TABLE images (
               img_idx int primary key,
               img_md5 varchar(32)
            )
            ");
      }

   } // check_config_table

   public function gen_thumbs($fromcmd = 0)
   {
      /* get all available photos */
      $all = $this->getAllTagPhotos();                                                                                                                                                                    
      foreach($all as $photo) {

         $full_path = $this->translate_path($details['directory_path'])  ."/". $details['name'];
         $file_md5 = md5_file($full_path);

         $details = $this->get_photo_details($photo);
         if($fromcmd) print "Image ". $details['name'] ." Thumbnails:";

         /* if the file hasn't changed there is no need to regen the thumb */
         if($file_md5 == $this->getMD5($photo)) {
            if($fromcmd) print " file has not changed - skipping\n";
            continue;
         }

         /* set the new/changed MD5 sum for the current photo */
         $this->setMD5($photo, $file_md5);

         /* create thumbnails for the requested resolutions */
         foreach(Array($this->cfg->thumb_width, $this->cfg->photo_width) as $resolution) {
            if($fromcmd) print " ". $resolution ."px";
            $this->create_thumbnail($full_path, $resolution);
         }

         if($fromcmd) print "\n";

      }

   } // gen_thumbs()

   private function getMD5($idx)
   {
      $result = $this->cfg_db->db_query("
         SELECT img_md5 
         FROM images
         WHERE img_idx='". $idx ."'
      ");

      if(!$result)
         return 0;

      $img = $this->cfg_db->db_fetch_object($result);
      return $img['img_md5'];
      
   } // getMD5()

   private function setMD5($idx, $md5)
   {
      $result = $this->cfg_db->db_exec("
         REPLACE INTO images (img_idx, img_md5)
         VALUES ('". $idx ."', '". $md5 ."')
      ");

   } // setMD5()

}

?>
